/*
 *	 LM75 auslesen; auf SAA1064 ausgeben
 *  ====================================
 *          Uwe Berger, 2012
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <linux/i2c-dev.h>
#include <sys/ioctl.h>

/* I2C-Definitionen */
#define DEV_I2C		"/dev/i2c-2"
#define ADDR_LM75	0x49
#define ADDR_7SEGM  0x38

/* Zahl --> 7-Segment-Anzeige */
const unsigned char segm[] = {0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F};
const unsigned char point  = 0x80;

#define DO_ERROR	{printf("Error...!\n"); exit(1);}

int  file;
unsigned char buffer[2], sign, dec;

/************************************/
void saa1064_write(int d1, int d2, int d3, int d4) {
	unsigned char buf[6];
	buf[0] = 0x00;	/* Basisadresse */
  	buf[1] = 0x77;	/* Init: 21mA; alle 4 Digits...  */
	buf[2] = d1;	/* Digit 1 */
	buf[3] = d2;	/* Digit 2 */
	buf[4] = d3;	/* Digit 3 */
	buf[5] = d4;	/* Digit 4 */
	if (ioctl(file, I2C_SLAVE, ADDR_7SEGM)) DO_ERROR;
	if (write(file, &buf, 6) != 6) DO_ERROR;
}


/************************************/
/************************************/
/************************************/
int main(void) {
	
	/* Device oeffnen */
	file = open(DEV_I2C, O_RDWR);
	if (file < 0) DO_ERROR;
	
	/* Temperatur lesen */
	if (ioctl(file, I2C_SLAVE, ADDR_LM75)) DO_ERROR;
	buffer[0] = 0x00;
	if (write(file, buffer, 1) != 1) DO_ERROR;
	if (read(file, buffer, 2) != 2)  DO_ERROR;
	
	/* Temperatur interpretieren/ausgeben */
	if (buffer[0] & 0b10000000) sign = 0x40; else sign = 0x00;
	if (buffer[1] & 0b10000000) dec = segm[5]; else dec = segm[0];	
	buffer[0]=buffer[0] & 0b01111111;
	saa1064_write(sign, segm[buffer[0]/10], segm[buffer[0]%10]+point, dec);

	/* Ende */	
	close(file);
	exit(0);
}
